module Asg
  module Rebooter
    module Configuration
      class AutoScalingGroup
        ASG_MIN_SIZE_DELTA = 1
        AZREBALANCE = 'AZRebalance'

        attr_reader :asg, :original_min_size, :original_suspended_processes

        def initialize(asg)
          @asg = asg

          set_original_values
        end

        def prepare
          suspend_az_rebalance
          decrease_min_size
        end

        def restore
          restore_az_rebalance
          restore_min_size
        end

        private

        def set_original_values
          @original_min_size = asg.min_size
          @original_suspended_processes = asg.suspended_processes
        end

        def suspend_az_rebalance
          return unless adjust_az_rebalance?

          asg.suspend_processes([AZREBALANCE])
        end

        def restore_az_rebalance
          return unless adjust_az_rebalance?

          asg.resume_processes([AZREBALANCE])
        end

        def decrease_min_size
          return unless adjust_min_size?

          new_min_size = @original_min_size - ASG_MIN_SIZE_DELTA

          asg.update_min_size(new_min_size)
        end

        def restore_min_size
          return unless adjust_min_size?

          asg.update_min_size(original_min_size)
        end

        def adjust_min_size?
          @original_min_size > 0
        end

        def adjust_az_rebalance?
          !original_suspended_processes.include?(AZREBALANCE)
        end
      end
    end
  end
end
