module Asg
  module Rebooter
    class InstanceRebooter
      REBOOT_TIMEOUT = 60

      attr_reader :asg, :asg_config, :target_group_config

      def initialize(asg)
        @asg = asg
      end

      def execute
        puts "[start] Starting reboot sequence for: #{asg.name}"

        begin
          prepare_configuration

          instances.each do |instance|
            puts "#{'*' * 4} #{instance.instance_id} #{'*' * 4}"
            puts "#{instance.instance_id} [enter_standby]"
            instance.enter_standby

            puts "#{instance.instance_id} [reboot]"
            instance.reboot

            sleep REBOOT_TIMEOUT # TODO: Ensure instance is rebooted: how to do this?

            puts "#{instance.instance_id} [exit_standby]"
            instance.exit_standby

            puts "#{instance.instance_id} [wait_for_healthy]"
            target_group.wait_for_healthy(instance)

            puts "\n"
          end
        ensure
          restore_configuration
        end
      end

      private

      def instances
        asg.instances
      end

      def target_group
        asg.target_groups.first
      end

      def prepare_configuration
        @asg_config = Asg::Rebooter::Configuration::AutoScalingGroup.new(asg)

        @target_group_config = asg.target_groups.map do |target_group|
          Asg::Rebooter::Configuration::TargetGroup.new(target_group)
        end

        asg_config.prepare
        target_group_config.each(&:prepare)
      end

      def restore_configuration
        asg_config.restore
        target_group_config.each(&:restore)
      end
    end
  end
end
