module Asg
  module Rebooter
    module Wrappers
      class AutoScalingGroupNameRequiredError < StandardError; end

      class AutoScalingGroup
        attr_reader :name

        def initialize(name)
          raise AutoScalingGroupNameRequiredError unless name

          @name = name
        end

        def suspended_processes
          asg.suspended_processes
        end

        def suspend_processes(processes)
          asg.suspend_processes(scaling_processes: processes)
        end

        def resume_processes(processes)
          asg.resume_processes(scaling_processes: processes)
        end

        def min_size
          asg.min_size
        end

        def update_min_size(new_min_size)
          asg.update(min_size: new_min_size)
        end

        def target_groups
          asg.target_group_arns.map do |arn|
            Asg::Rebooter::Wrappers::TargetGroup.new(arn)
          end
        end

        def instances
          asg.instances.map do |instance|
            Asg::Rebooter::Wrappers::Instance.new(instance)
          end
        end

        def reboot
          Asg::Rebooter::InstanceRebooter.new(self).execute
        end

        private

        def asg
          Aws::AutoScaling::AutoScalingGroup.new(name)
        end
      end
    end
  end
end
