module Asg
  module Rebooter
    module Wrappers
      class TargetGroupArnRequiredError < StandardError; end

      class TargetGroup
        TARGET_GROUP_INSTANCE_HEALTHY = 'healthy'
        POLL_TIMEOUT = 4

        attr_reader :arn

        def initialize(arn)
          raise TargetGroupArnRequiredError unless arn

          @arn = arn
        end

        def attributes
          target_group_attributes = client.describe_target_group_attributes(target_group_arn: arn)
          target_group_attributes.attributes.map(&:to_hash)
        end

        def restore_attributes
          update_attributes(original_attributes)
        end

        def update_attributes(attributes_to_update)
          client.modify_target_group_attributes \
            target_group_arn: arn,
            attributes: attributes_to_update
        end

        def target_health_descriptions
          client.describe_target_health(target_group_arn: arn).target_health_descriptions
        end

        def health_for_instance(instance)
          #
          # TODO: Write specs and error handling
          #
          health_description_for_instance = target_health_descriptions.keep_if do |target_health_description|
            target_health_description.target.id == instance.id
          end

          if health_description_for_instance.any?
            health_description_for_instance.first.target_health.state
          else
            'unknown'
          end
        end

        def wait_for_healthy(instance)
          loop do
            healthy = health_for_instance(instance) == TARGET_GROUP_INSTANCE_HEALTHY

            break if healthy

            sleep(POLL_TIMEOUT)
          end

          true
        end

        private

        def client
          @client ||= Aws::ElasticLoadBalancingV2::Client.new
        end
      end
    end
  end
end
